#!/usr/bin/env python3

import codecs
import json

import grpc
import requests

import deepmine.speech.v1.speech_pb2 as speech_pb2
import deepmine.speech.v1.speech_pb2_grpc as speech_pb2_grpc
from main.utils import levenshtein

CHUNK_SIZE = 4000
MAX_MESSAGE_LENGTH = 128 * 1024 * 1024  # 128 MB
# SERVER_ADDRESS = '185.173.105.83:8444'
SERVER_ADDRESS = 'grpc2.deepmine.ir:443'
# SERVER_ADDRESS = 'deepmine.ir:8443'
GATEWAY_BASE_URL = 'https://api2.deepmine.ir/v1'
USER_NAME = '09125225483'
PASSWORD = 'Shvh1348_'


verbs = set()
with codecs.open('verbs.txt', 'r', 'utf8') as _fid:
    for line in _fid:
        verbs.add(line.strip())

# with open('cert/fullchain.pem', 'rb') as fid:
with open('cert/server.crt', 'rb') as fid:
    trusted_certs = fid.read()


def api_login(username, password):
    data = {'username': username, 'password': password}
    try:
        response = requests.post(GATEWAY_BASE_URL + '/user/login/', data=data)
        response.raise_for_status()
        output = json.loads(response.content)
        # print(output)
        return output.get('access'), output.get('refresh')
    except requests.exceptions.HTTPError as error:
        print(error)
        return None, None


# class GrpcAuth(grpc.AuthMetadataPlugin):
#     def __init__(self, key):
#         self._key = key
#
#     def __call__(self, context, callback):
#         callback((('access-token', self._key),), None)


def streaming_recognize_generator(audio_file_name):
    config = speech_pb2.RecognitionConfig(
        encoding='LINEAR16',
        language_code='fa-IR',
        enable_word_confidence=True,
        max_alternatives=1,
        sample_rate_hertz=16000,
        model='dictation'
    )

    streaming_config = speech_pb2.StreamingRecognitionConfig(config=config,
                                                             single_utterance=False,
                                                             interim_results=True,
                                                             interim_results_interval_milliseconds=500)

    yield speech_pb2.StreamingRecognizeRequest(streaming_config=streaming_config)

    with open(audio_file_name, 'rb') as f:
        # f.read(44)
        data = f.read(CHUNK_SIZE)
        while data != b'':
            yield speech_pb2.StreamingRecognizeRequest(audio_content=data)
            data = f.read(CHUNK_SIZE)


def recognize_request(audio_file_name):
    # audio_file_name = r'samples\zeinali_ar_16000.wav'
    config = speech_pb2.RecognitionConfig(
        encoding='LINEAR16',
        language_code='fa-IR',
        enable_word_confidence=True,
        sample_rate_hertz=16000,
        model='dictation'
    )

    with open(audio_file_name, 'rb') as fid:
        recognition_audio = speech_pb2.RecognitionAudio(content=fid.read())
        # recognition_audio = speech_pb2.RecognitionAudio(uri='https://deepmine.ir/taeb1.wav')

    return speech_pb2.RecognizeRequest(config=config, audio=recognition_audio)


def align_request(audio_file_name, text):
    # print('Starting')
    config = speech_pb2.RecognitionConfig(
        encoding='LINEAR16',
        language_code='en-EN',
        enable_word_confidence=True,
        sample_rate_hertz=16000
    )

    with open(audio_file_name, 'rb') as fid:
        recognition_audio = speech_pb2.RecognitionAudio(content=fid.read())

    return speech_pb2.AlignmentRequest(config=config, audio=recognition_audio, text=text)


def run_bidi_streaming_recognize(audio_file_name):
    try:
        credentials = grpc.ssl_channel_credentials(root_certificates=trusted_certs)

        access, _ = api_login(USER_NAME, PASSWORD)
        # credentials = grpc.composite_channel_credentials(
        #     credentials,
        #     grpc.metadata_call_credentials(
        #         GrpcAuth(access)
        #     )
        # )

        channel = grpc.secure_channel(
            SERVER_ADDRESS,
            credentials,
            options=[
                ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
                ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
            ],
        )

        stub = speech_pb2_grpc.SpeechStub(channel)
        generator = streaming_recognize_generator(audio_file_name)
        it = stub.BidiStreamingRecognize(generator, metadata=[('access-token', access)])

        for r in it:
            try:
                print(speech_pb2.SpeechEventType.Name(r.speech_event_type))
                for result in r.results:
                    is_final = result.is_final
                    is_the_first_alternative = True
                    for alternative in result.alternatives:
                        print('Alternative:', alternative.transcript)
                        if is_final:
                            print('Alternative confidence: %.4f' % alternative.confidence)
                            # print('Words:')
                            # for word in alternative.words:
                            #     print('%s %-6.3f %-6.3f %.3f' % (word.word, word.start_time,
                            #                                      word.end_time, word.confidence))
                        elif is_the_first_alternative:
                            is_the_first_alternative = False
                            print('Alternative stability: %.3f' % result.stability)
                        else:
                            print('Is final: ', is_final)
                print('--------------- END CHUNK ---------------')
            except LookupError:
                print('No available chunks')
    except grpc.RpcError as err:
        print('Error code %s\nError message: %s' % (err.code(), err.details()))


def _streaming_recognize(audio_file_name):
    access, _ = api_login(USER_NAME, PASSWORD)
    credentials = grpc.ssl_channel_credentials(root_certificates=trusted_certs)

    # credentials = grpc.composite_channel_credentials(
    #     credentials,
    #     grpc.metadata_call_credentials(
    #         GrpcAuth(access)
    #     )
    # )

    channel = grpc.secure_channel(
        SERVER_ADDRESS,
        credentials,
        options=[
            ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
            ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
        ],
    )

    stub = speech_pb2_grpc.SpeechStub(channel)

    generator = streaming_recognize_generator(audio_file_name)
    recognize_response = stub.StreamingRecognize(generator, metadata=[('access-token', access)])

    return recognize_response


def _recognize(audio_file_name):
    access, _ = api_login(USER_NAME, PASSWORD)
    credentials = grpc.ssl_channel_credentials(root_certificates=trusted_certs)

    # credentials = grpc.composite_channel_credentials(
    #     credentials,
    #     grpc.metadata_call_credentials(
    #         GrpcAuth(access)
    #     )
    # )

    channel = grpc.secure_channel(
        SERVER_ADDRESS,
        credentials,
        options=[
            ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
            ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
        ],
    )

    stub = speech_pb2_grpc.SpeechStub(channel)
    recognize_response = stub.Recognize(recognize_request(audio_file_name),
                                        metadata=[('access-token', access)])

    return recognize_response


def _print_recognize_response(recognize_response):
    for result in recognize_response.results:
        print('Event:', speech_pb2.SpeechEventType.Name(result.speech_event_type))
        for alternative in result.alternatives:
            print('Alternative:', alternative.transcript)
            print('Alternative confidence: %.3f' % alternative.confidence)
            print('Words:')
            for word in alternative.words:
                print('%s %-6.3f %-6.3f %.3f' % (word.word, word.start_time, word.end_time, word.confidence))
        print('--------------- END CHUNK ---------------\n')


def run_recognize(audio_file_name):
    try:
        recognize_response = _recognize(audio_file_name)
        _print_recognize_response(recognize_response)
    except grpc.RpcError as err:
        print('Error code %s\nError message: %s' % (err.code(), err.details()))


def run_streaming_recognize(audio_file_name):
    try:
        recognize_response = _streaming_recognize(audio_file_name)
        _print_recognize_response(recognize_response)
    except grpc.RpcError as err:
        print('Error code %s\nError message: %s' % (err.code(), err.details()))


def _generate_initial_segments(recognize_response):
    initial_segments = []
    min_segment_words = 4
    max_allowed_silence = 0.6
    max_allowed_silence_after_verb = 0.4

    def add_new_segment(_words):
        start_time = _words[0].start_time
        end_time = _words[-1].end_time
        confidence = 0
        temp_words = []
        for _word in _words:
            temp_words.append(_word.word)
            confidence += _word.confidence
        confidence /= len(temp_words)
        initial_segments.append([' '.join(temp_words), start_time, end_time, confidence])

    for result in recognize_response.results:
        for alternative in result.alternatives:
            words = []
            verb_seen = False
            last_word_end_time = 1000000  # an invalid time
            count = len(alternative.words)
            for i, word in enumerate(alternative.words):
                if word.word in verbs:
                    words.append(word)
                    verb_seen = True
                else:
                    flag = word.start_time - last_word_end_time >= max_allowed_silence
                    flag_verb = word.start_time - last_word_end_time >= max_allowed_silence_after_verb
                    if len(words) >= min_segment_words and count - i >= min_segment_words:
                        if (verb_seen and flag_verb) or flag:
                            add_new_segment(words)
                            words = []
                    elif verb_seen and flag_verb:
                        add_new_segment(words)
                        words = []
                    words.append(word)
                    verb_seen = False
                last_word_end_time = word.end_time

            if len(words) > 0:
                add_new_segment(words)

    return initial_segments


def _generate_final_segments(initial_segments):
    final_segments = []
    min_allowed_silence = 0.15
    i, _len = 0, len(initial_segments)
    while i < _len:
        segment = initial_segments[i][:]
        segment[1] = max(0, segment[1] - 0.1)
        if i < _len - 1 and initial_segments[i+1][1] - segment[2] > 0.15:
            segment[2] += 0.1
        if final_segments and segment[1] - final_segments[-1][2] < min_allowed_silence:
            last_segment = final_segments.pop()
            new_segment = [last_segment[0] + ' ' + segment[0], last_segment[1], segment[2],
                           (segment[-1] + last_segment[-1]) / 2]
            final_segments.append(new_segment)
        else:
            final_segments.append(segment)
        i += 1
    return final_segments


def generate_segments(audio_file_name):
    try:
        recognize_response = _streaming_recognize(audio_file_name)
        initial_segments = _generate_initial_segments(recognize_response)
        final_segments = _generate_final_segments(initial_segments)
        for segment in final_segments:
            print(segment)
    except grpc.RpcError as err:
        print('Error code %s\nError message: %s' % (err.code(), err.details()))


def run_aligner(audio_file_name):
    channel = grpc.insecure_channel(
        SERVER_ADDRESS,
        options=[
            ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
            ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
        ],
    )
    stub = speech_pb2_grpc.SpeechStub(channel)
    # print('step 3')
    with codecs.open(audio_file_name + '.txt', 'r', encoding='utf8') as fid:
        text = fid.read()
    response = stub.Align(align_request(audio_file_name, text))
    print('step 4')
    try:
        all_words = text.split()
        recognized_words = []
        for word in response.words:
            recognized_words.append(word.word)
            print('%s - [ %.3f - %.3f ]  %.2f' % (word.word, word.start_time, word.end_time, word.confidence))
        _, cost = levenshtein(all_words, recognized_words)
        print('Mismatch count:', cost)
    except grpc.RpcError as err:
        print('Error code %s\nError message: %s' % (err.code(), err.details()))


if __name__ == '__main__':
    # parser = argparse.ArgumentParser()
    # parser.add_argument('--path', required=True, help='audio file path')
    # args = parser.parse_args()

    run_recognize(r'samples\zeinali_48000.wav')#args.path)
    run_streaming_recognize(r'samples\test_fa.wav')
    # run_bidi_streaming_recognize(r'samples\test_fa.wav')
    # run_aligner(r'D:\Downloads\Akhlagh 2 - 2.wav')
    # generate_segments(r'D:\Databases\TTS_Example\AUD-20201026-WA0007_darvishi.wav')#args.path)
    # generate_segments(r'D:\Downloads\D1737735T17180028(Web).wav')#args.path)
    # generate_segments(r'D:\Downloads\Akhlagh 2 - 2.wav')#args.path)
