import requests
import urllib3

from main import status
from main.exceptions import InvalidToken, HTTPError
from settings import API_GATEWAY_BASE_URL, GRPC_SECRET_HEADERS, API_TIMEOUT_SEC

urllib3.disable_warnings()


def api_get_remaining_quota(access_token):
    headers = {'Authorization': 'Token ' + access_token}
    try:
        response = requests.get(API_GATEWAY_BASE_URL + '/plans/user/active/', headers=headers, verify=False,
                                timeout=API_TIMEOUT_SEC)
    except requests.exceptions.Timeout:
        return None
    if response.status_code == status.HTTP_401_UNAUTHORIZED:
        raise InvalidToken()
    if response.status_code >= 400:
        raise HTTPError(status_code=response.status_code, reason=response.reason, api_message=response.text)
    return response.json()


def api_get_remaining_quota_by_user_id(user_id):
    try:
        response = requests.get(API_GATEWAY_BASE_URL + '/plans/user/active/admin/?user=%s' % str(user_id),
                                headers=GRPC_SECRET_HEADERS, verify=False, timeout=API_TIMEOUT_SEC)
    except requests.exceptions.Timeout:
        return None
    if response.status_code == status.HTTP_401_UNAUTHORIZED:
        raise InvalidToken()
    if response.status_code >= 400:
        raise HTTPError(status_code=response.status_code, reason=response.reason, api_message=response.text)
    return response.json()


def api_use_quota(user_id, basic_used_sec, nonbasic_used_sec):
    data = {'user_id': user_id, 'basic_used_sec': basic_used_sec, 'nonbasic_used_sec': nonbasic_used_sec}
    try:
        response = requests.post(API_GATEWAY_BASE_URL + '/plans/user/use/', data=data, verify=False,
                                 headers=GRPC_SECRET_HEADERS, timeout=API_TIMEOUT_SEC)
    except requests.exceptions.Timeout:
        return None
    if response.status_code >= 400:
        # raise HTTPError(status_code=response.status_code, reason=response.reason, api_message=response.text)
        return None
    return response.json()


def api_login(username, password):
    data = {'username': username, 'password': password}
    response = requests.post(API_GATEWAY_BASE_URL + '/user/login/', verify=False, data=data)
    if response.status_code == status.HTTP_401_UNAUTHORIZED:
        raise InvalidToken()
    if response.status_code >= 400:
        raise HTTPError(status_code=response.status_code, reason=response.reason, api_message=response.text)
    output = response.json()
    return output.get('access'), output.get('refresh')


def test_ap():
    headers = {'Authorization': 'DxcBYFKl75kO7cDO88OOksugRKQPSJPnYz7ISa1ksodKUfoxslypDJyjD91WDgaAF7sWEqmYT2X8lkWJB1MYYwJwDEFMz9NKLX1Z8PXBtj9JF48gwSvqUD2ptYzPs5So'}
    # response = requests.get('https://dadegan.deepmine.ir/api/utterance/1/home?record_date=2022-08-24%2022:38:25%2b03:30&duration=123&duration_without_sil=104&is_conversational=0', headers=headers, verify=False)
    response = requests.get('https://dadegan.deepmine.ir/api/contribution/09125225483', headers=headers, verify=False)

    print(response.json())
    return response.json()


if __name__ == '__main__':
    # sys.stdout.reconfigure(encoding='utf-8')
    # access, refresh = api_login('09125225483', 'Shvh1348_')
    # print(access)
    # print(api_get_remaining_quota(access))
    test_ap()
