import numpy as np
import requests

from settings import FILE_ACCESS_ADMIN_TOKEN


def levenshtein(seq1, seq2):
    size_x = len(seq1) + 1
    size_y = len(seq2) + 1
    matrix = np.zeros((size_x, size_y), dtype=np.int32)
    matrix[:, 0] = np.arange(size_x)
    matrix[0, :] = np.arange(size_y)

    diff = 10 + abs(size_x - size_y)
    for i in range(1, size_x):
        for j in range(1, size_y):
            if i - diff < j < i + diff:
                matrix[i, j] = min(matrix[i - 1, j] + 1,
                                   matrix[i - 1, j - 1] + (seq1[i - 1] != seq2[j - 1]),
                                   matrix[i, j - 1] + 1)
            else:
                matrix[i, j] = matrix[i - 1, j - 1] + 1

    edits = []
    i, j = len(seq1), len(seq2)
    while not (i == 0 and j == 0):
        prev_cost = matrix[i, j]
        neighbors = []
        if i != 0 and j != 0:
            neighbors.append(matrix[i - 1, j - 1])
        if i != 0:
            neighbors.append(matrix[i - 1, j])
        if j != 0:
            neighbors.append(matrix[i, j - 1])

        min_cost = min(neighbors)
        if min_cost == prev_cost:  # match
            i, j = i - 1, j - 1
            edits.append(('m', i, j))
        elif i != 0 and j != 0 and min_cost == matrix[i - 1, j - 1]:  # substitution
            i, j = i - 1, j - 1
            edits.append(('s', i, j))
        elif i != 0 and min_cost == matrix[i - 1, j]:  # deletion
            i, j = i - 1, j
            edits.append(('d', i, j))
        elif j != 0 and min_cost == matrix[i, j - 1]:  # insertion
            i, j = i, j - 1
            edits.append(('i', i, j))

    edits.reverse()
    return edits, matrix[-1, -1]


def download_file(url):
    try:
        if 'deepmine.ir' in url:
            headers = {'Authorization': 'Token ' + FILE_ACCESS_ADMIN_TOKEN}
            response = requests.get(url, allow_redirects=True, headers=headers)
        else:
            response = requests.get(url, allow_redirects=True)
        response.raise_for_status()
        return response.content, None
    except requests.exceptions.HTTPError as err:
        return None, str(err)


if __name__ == '__main__':
    # import codecs, timeit
    # with codecs.open(r'D:\Downloads\Akhlagh 2 - 2.wav.txt', 'r', encoding='utf8') as fid:
    #     word_seq1 = fid.read().split()
    # with codecs.open(r'D:\Downloads\Akhlagh 2 - 2.wav_asr.txt', 'r', encoding='utf8') as fid:
    #     word_seq2 = fid.read().split()
    # # print(timeit.timeit('levenshtein(word_seq1, word_seq2)', globals=globals(), number=500))
    # edits, cost = levenshtein(word_seq1, word_seq2)
    # for edit in edits:
    #     print(edit[0], word_seq1[edit[1]], word_seq2[edit[2]])
    # print(costs)
    content = download_file('https://file-access.deepmine.ir/file?path=content/2ba57f6e-a25')
    with open("temp2.html", 'wb') as fid:
        fid.write(content[0])
    # print(content[0].decode('UTF-8'))
    # import os
    # print(os.path.splitext(os.path.basename('d1\\d2\\example')))
    # import json
    # json.loads("Received http2 header with status: 504")
    pass

